﻿using System;
using System.Collections.Generic;
using System.Text;
using Squid;
using Framework;
using SampleControls;

namespace SquidSlimDX
{
    public class SampleScene : DrawableGameComponent
    {
        private SampleDesktop Desktop;

        public SampleScene(Game game)
            : base(game)
        {
        }

        public override void Load()
        {
            Desktop = new SampleDesktop { Name = "desk" };
            Desktop.ShowCursor = true;

            // -- Uncomment to load and apply the style texture
            MapStyles("SampleMap");

            base.Load();
        }

        public override void Draw(GameTime time)
        {
            Desktop.Size = new Point(Game.Device.Viewport.Width, Game.Device.Viewport.Height);
            Desktop.Update();
            Desktop.Draw();

            int tex = GuiHost.Renderer.GetTexture("slimdx_logo.png");
            GuiHost.Renderer.StartBatch();
            GuiHost.Renderer.DrawTexture(tex, Game.Device.Viewport.Width - 130, Game.Device.Viewport.Height - 130, 128, 128, new UVCoords(0, 0, 1, 1), -1);
            GuiHost.Renderer.EndBatch(true);
        }

        private void MapStyles(string mapName)
        {
            StyleMap map = new StyleMap();
            map.LoadFile(AppDomain.CurrentDomain.BaseDirectory + "Content\\" + mapName + ".xml");

            foreach (ControlStyle style in GuiHost.GetSkin().Styles.Values)
            {
                foreach (Style state in style.Styles.Values)
                {
                    if (string.IsNullOrEmpty(state.Texture))
                        continue;

                    if (map.Contains(state.Texture))
                    {
                        state.TextureUV = map.GetUVCoords(state.Texture);
                        state.Texture = mapName + ".png";
                    }
                }
            }
        }
    }
}
