﻿using System;
using System.Collections.Generic;
using System.Text;
using Squid;
using Framework;
using SampleControls;

namespace SquidTV3D
{
    public class SampleScene : DrawableGameComponent
    {
        private SampleDesktop Desktop;

        public SampleScene(Game game)
            : base(game)
        {
        }

        public override void Load()
        {
            //Game.Engine.EnableProfiler(true);
            Game.Engine.ShowWinCursor(false);

            Desktop = new SampleDesktop { Name = "desk" };
            Desktop.ShowCursor = true;

            // -- Uncomment to create the style texture
            //PackTextures("SampleMap", 6);

            // -- Uncomment to load and apply the style texture
            MapStyles("SampleMap");

            //string data = new Squid.Xml.XmlSerializer().Serialize(Desktop);
            //System.IO.File.WriteAllText("E:\\testsave.xml", data);

            base.Load();
        }

        public override void Draw(GameTime time)
        {
            Desktop.Size = new Squid.Point(Game.Engine.GetViewport().GetWidth(), Game.Engine.GetViewport().GetHeight());
            Desktop.Update();
            Desktop.Draw();
        }

        private void CreateAtlas()
        {

        }

        /// <summary>
        /// This method scans all registered styles,
        /// grabs their textures and merges them into a single output texture.
        /// It also adjusts the UV coordinates of the styles accordingly.
        /// The result is an XML file that maps style.Texture to a section of the newly created output texture.
        /// </summary>
        /// <param name="mapName"></param>
        /// <param name="spacing"></param>
        private void PackTextures(string mapName, int spacing)
        {
            StyleMap map = new StyleMap();
            int textureSize = 512;
            int dest = Game.Textures.CreateTexture(textureSize, textureSize);
            Squid.RectanglePacker packer = new Squid.RectanglePacker(textureSize, textureSize);
            Dictionary<string, bool> textures = new Dictionary<string, bool>();

            foreach (ControlStyle style in GuiHost.GetSkin().Styles.Values)
            {
                foreach (Style state in style.Styles.Values)
                {
                    if (string.IsNullOrEmpty(state.Texture))
                        continue;

                    string[] filepath = System.IO.Directory.GetFiles(AppDomain.CurrentDomain.BaseDirectory, state.Texture, System.IO.SearchOption.AllDirectories);
                    if (filepath.Length < 1)
                        continue;

                    if (map.Contains(state.Texture))
                        continue;

                    string name = System.IO.Path.GetFileName(filepath[0]);
                    int source = GuiHost.Renderer.GetTexture(name);

                    MTV3D65.TV_TEXTURE info = Game.Textures.GetTextureInfo(source);
                    Squid.Point p = packer.Pack(info.Width + spacing, info.Height + spacing);

                    Game.Textures.LockTexture(source, true);
                    Game.Textures.LockTexture(dest);

                    int[] pixels = new int[info.Width * info.Height];
                    Game.Textures.GetPixelArray(source, 0, 0, info.Width, info.Height, pixels);
                    Game.Textures.SetPixelArray(dest, p.x, p.y, info.Width, info.Height, pixels);

                    Game.Textures.UnlockTexture(source);
                    Game.Textures.UnlockTexture(dest);

                    UVCoords uv = new UVCoords();
                    uv.U1 = (float)p.x / (float)textureSize;
                    uv.V1 = (float)p.y / (float)textureSize;
                    uv.U2 = (float)(p.x + info.Width) / (float)textureSize;
                    uv.V2 = (float)(p.y + info.Height) / (float)textureSize;

                    map.Add(state.Texture, uv);
                }
            }

            string stub = AppDomain.CurrentDomain.BaseDirectory + "Content\\" + mapName;
            Game.Textures.SaveTexture(dest, stub + ".png", MTV3D65.CONST_TV_IMAGEFORMAT.TV_IMAGE_PNG);
            map.Save(stub + ".xml");
        }

        private void MapStyles(string mapName)
        {
            StyleMap map = new StyleMap();
            map.LoadFile(AppDomain.CurrentDomain.BaseDirectory + "Content\\" + mapName + ".xml");

            foreach (ControlStyle style in GuiHost.GetSkin().Styles.Values)
            {
                foreach (Style state in style.Styles.Values)
                {
                    if (string.IsNullOrEmpty(state.Texture))
                        continue;

                    if (map.Contains(state.Texture))
                    {
                        state.TextureUV = map.GetUVCoords(state.Texture);
                        state.Texture = mapName + ".png";
                    }
                }
            }
        }
    }
}
